import 'package:flutter/material.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:syncfusion_flutter_core/core.dart';
import 'package:syncfusion_flutter_datepicker/datepicker.dart';
import 'package:syncfusion_flutter_datepicker/src/date_picker/picker_helper.dart';

void main() {
  dateTimeEngineTest();
  hijriCalendarHelperTest();
  selectablePredicates();
  hijriSelectablePredicates();
}

class SelectableDayPredicate extends StatefulWidget {
  @override
  // ignore: library_private_types_in_public_api
  _SelectableDayPredicateState createState() => _SelectableDayPredicateState();
}

class _SelectableDayPredicateState extends State<SelectableDayPredicate> {
  int count = 0;
  int selectionChangedCount = 0;

  final DateRangePickerController _pickerController =
      DateRangePickerController();
  final DateTime _initialDisplayDate = DateTime(2019, 02, 13);
  final DateTime _initialSelectedDate = DateTime(2019, 02, 13);
  bool _allowViewNavigation = true;
  bool _showTrailingAndLeadingDates = false;
  bool _enableMultiView = false;

  void selectionChanged(DateRangePickerSelectionChangedArgs args) {
    selectionChangedCount++;
  }

  @override
  Widget build(BuildContext context) {
    return MaterialApp(
        home: Scaffold(
            appBar: AppBar(
              title: const Text('Selectable day predicates',
                  textAlign: TextAlign.center,
                  softWrap: true,
                  textScaler: TextScaler.noScaling,
                  overflow: TextOverflow.fade,
                  style: TextStyle(
                    fontSize: 24.0,
                    color: Colors.white,
                  )),
              centerTitle: true,
              backgroundColor: Colors.blue,
            ),
            endDrawer: SafeArea(
              child: Drawer(child: _getPropertyView()),
            ),
            body: _getDatePicker()));
  }

  Widget _getPropertyView() {
    return Column(children: <Widget>[
      Row(
        mainAxisAlignment: MainAxisAlignment.spaceEvenly,
        children: <Widget>[
          Expanded(
            child: MaterialButton(
                key: const Key('Reset'),
                child: const Text('Reset'),
                onPressed: () {
                  setState(() {});
                }),
          ),
          Expanded(
            child: MaterialButton(
                key: const Key('month'),
                child: const Text('Month'),
                onPressed: () {
                  _pickerController.view = DateRangePickerView.month;
                }),
          ),
          Expanded(
            child: MaterialButton(
                key: const Key('year'),
                child: const Text('Year'),
                onPressed: () {
                  count = 0;
                  _pickerController.view = DateRangePickerView.year;
                }),
          ),
        ],
      ),
      Row(
        mainAxisAlignment: MainAxisAlignment.spaceEvenly,
        children: <Widget>[
          Expanded(
            child: MaterialButton(
                key: const Key('decade'),
                child: const Text('Decade'),
                onPressed: () {
                  count = 0;
                  _pickerController.view = DateRangePickerView.decade;
                }),
          ),
          Expanded(
            child: MaterialButton(
                key: const Key('century'),
                child: const Text('Century'),
                onPressed: () {
                  count = 0;
                  _pickerController.view = DateRangePickerView.century;
                }),
          ),
        ],
      ),
      Row(
        mainAxisAlignment: MainAxisAlignment.spaceEvenly,
        children: <Widget>[
          Expanded(
            child: MaterialButton(
                elevation: 0,
                focusElevation: 0,
                highlightElevation: 0,
                disabledElevation: 0,
                hoverElevation: 0,
                key: const Key('showTrailingAndLeadingDates'),
                child: const Text('TrailAndLeadDates'),
                onPressed: () {
                  setState(() {
                    count = 0;
                    _showTrailingAndLeadingDates = true;
                  });
                }),
          ),
        ],
      ),
      Row(
        mainAxisAlignment: MainAxisAlignment.spaceEvenly,
        children: <Widget>[
          Expanded(
            child: MaterialButton(
                elevation: 0,
                focusElevation: 0,
                highlightElevation: 0,
                disabledElevation: 0,
                hoverElevation: 0,
                key: const Key('hideTrailingAndLeadingDates'),
                child: const Text('HideTrailAndLeadDates'),
                onPressed: () {
                  setState(() {
                    count = 0;
                    _showTrailingAndLeadingDates = false;
                  });
                }),
          ),
        ],
      ),
      Row(
        mainAxisAlignment: MainAxisAlignment.spaceEvenly,
        children: <Widget>[
          Expanded(
            child: MaterialButton(
                elevation: 0,
                focusElevation: 0,
                highlightElevation: 0,
                disabledElevation: 0,
                hoverElevation: 0,
                key: const Key('allowViewNavigation'),
                child: const Text('AllowViewNavigation'),
                onPressed: () {
                  setState(() {
                    _allowViewNavigation = false;
                  });
                }),
          ),
        ],
      ),
      Row(
        mainAxisAlignment: MainAxisAlignment.spaceEvenly,
        children: <Widget>[
          Expanded(
            child: MaterialButton(
                elevation: 0,
                focusElevation: 0,
                highlightElevation: 0,
                disabledElevation: 0,
                hoverElevation: 0,
                key: const Key('enableMultiView'),
                child: const Text('MultiView'),
                onPressed: () {
                  setState(() {
                    count = 0;
                    _enableMultiView = true;
                  });
                }),
          ),
        ],
      ),
    ]);
  }

  Widget _getDatePicker() {
    return Column(
      children: <Widget>[
        Container(
          height: 200,
          width: 1500,
          color: Colors.black12,
          child: Text(
            'predicate count: $count '
            'selection changed count: $selectionChangedCount',
            key: const Key('predicateCount'),
          ),
        ),
        SfDateRangePicker(
            controller: _pickerController,
            initialSelectedDate: _initialSelectedDate,
            initialDisplayDate: _initialDisplayDate,
            allowViewNavigation: _allowViewNavigation,
            enableMultiView: _enableMultiView,
            onSelectionChanged: selectionChanged,
            monthViewSettings: DateRangePickerMonthViewSettings(
                showTrailingAndLeadingDates: _showTrailingAndLeadingDates),
            selectableDayPredicate: (DateTime date) {
              count++;
              if (date == DateTime(2019) ||
                  date == DateTime(2019, 02, 5) ||
                  date == DateTime(2019, 2, 7) ||
                  date == DateTime(2019, 2, 13) ||
                  date == DateTime(2019, 2, 19) ||
                  date == DateTime(2019, 2, 22) ||
                  date == DateTime(2019, 2, 25) ||
                  date == DateTime(2019, 2, 28)) {
                return false;
              }
              return true;
            }),
      ],
    );
  }
}

class HijriSelectablePredicate extends StatefulWidget {
  @override
  // ignore: library_private_types_in_public_api
  _HijriSelectablePredicateState createState() =>
      _HijriSelectablePredicateState();
}

class _HijriSelectablePredicateState extends State<HijriSelectablePredicate> {
  int count = 0;
  int selectionChangedCount = 0;

  final HijriDatePickerController _hijriPickerController =
      HijriDatePickerController();
  final HijriDateTime _initialDisplayDate = HijriDateTime(1441, 4, 2);
  final HijriDateTime _initialSelectedDate = HijriDateTime(1441, 4, 2);
  bool _allowViewNavigation = true;
  bool _enableMultiView = false;

  void selectionChanged(DateRangePickerSelectionChangedArgs args) {
    selectionChangedCount++;
  }

  @override
  Widget build(BuildContext context) {
    return MaterialApp(
        home: Scaffold(
            appBar: AppBar(
              title: const Text('Hijri Selectable day predicates',
                  textAlign: TextAlign.center,
                  softWrap: true,
                  textScaler: TextScaler.noScaling,
                  overflow: TextOverflow.fade,
                  style: TextStyle(
                    fontSize: 24.0,
                    color: Colors.white,
                  )),
              centerTitle: true,
              backgroundColor: Colors.blue,
            ),
            endDrawer: SafeArea(
              child: Drawer(child: _getPropertyView()),
            ),
            body: _getHijriDatePicker()));
  }

  Widget _getPropertyView() {
    return Column(children: <Widget>[
      Row(
        mainAxisAlignment: MainAxisAlignment.spaceEvenly,
        children: <Widget>[
          Expanded(
            child: MaterialButton(
                key: const Key('Reset'),
                child: const Text('Reset'),
                onPressed: () {
                  setState(() {});
                }),
          ),
          Expanded(
            child: MaterialButton(
                key: const Key('month'),
                child: const Text('Month'),
                onPressed: () {
                  _hijriPickerController.view = HijriDatePickerView.month;
                }),
          ),
          Expanded(
            child: MaterialButton(
                key: const Key('year'),
                child: const Text('Year'),
                onPressed: () {
                  count = 0;
                  _hijriPickerController.view = HijriDatePickerView.year;
                }),
          ),
        ],
      ),
      Row(
        mainAxisAlignment: MainAxisAlignment.spaceEvenly,
        children: <Widget>[
          Expanded(
            child: MaterialButton(
                key: const Key('decade'),
                child: const Text('Decade'),
                onPressed: () {
                  count = 0;
                  _hijriPickerController.view = HijriDatePickerView.decade;
                }),
          ),
        ],
      ),
      Row(
        mainAxisAlignment: MainAxisAlignment.spaceEvenly,
        children: <Widget>[
          Expanded(
            child: MaterialButton(
                elevation: 0,
                focusElevation: 0,
                highlightElevation: 0,
                disabledElevation: 0,
                hoverElevation: 0,
                key: const Key('allowViewNavigation'),
                child: const Text('AllowViewNavigation'),
                onPressed: () {
                  setState(() {
                    _allowViewNavigation = false;
                  });
                }),
          ),
        ],
      ),
      Row(
        mainAxisAlignment: MainAxisAlignment.spaceEvenly,
        children: <Widget>[
          Expanded(
            child: MaterialButton(
                elevation: 0,
                focusElevation: 0,
                highlightElevation: 0,
                disabledElevation: 0,
                hoverElevation: 0,
                key: const Key('enableMultiView'),
                child: const Text('MultiView'),
                onPressed: () {
                  setState(() {
                    count = 0;
                    _enableMultiView = true;
                  });
                }),
          ),
        ],
      ),
    ]);
  }

  bool _selectableDayPredicateDates(HijriDateTime date) {
    count++;
    if (date.weekday == 6 ||
        date.weekday == 7 ||
        date == HijriDateTime(1441, 4, 6) ||
        date == HijriDateTime(1441, 4, 7) ||
        date == HijriDateTime(1441, 4, 13) ||
        date == HijriDateTime(1441, 4, 19) ||
        date == HijriDateTime(1441, 4, 22) ||
        date == HijriDateTime(1441, 4, 25) ||
        date == HijriDateTime(1441, 4, 28)) {
      return false;
    }
    return true;
  }

  Widget _getHijriDatePicker() {
    return Column(
      children: <Widget>[
        Container(
          height: 200,
          width: 1500,
          color: Colors.black12,
          child: Text(
            'predicate count: $count '
            'selection changed count: $selectionChangedCount',
            key: const Key('predicateCount'),
          ),
        ),
        SfHijriDateRangePicker(
            controller: _hijriPickerController,
            initialSelectedDate: _initialSelectedDate,
            initialDisplayDate: _initialDisplayDate,
            allowViewNavigation: _allowViewNavigation,
            enableMultiView: _enableMultiView,
            onSelectionChanged: selectionChanged,
            selectableDayPredicate: _selectableDayPredicateDates),
      ],
    );
  }
}

void selectablePredicates() {
  group('SelectableDayPredicate', () {
    testWidgets(
        'SelectableDayPredicate triggers count must be 28 for month view',
        (WidgetTester tester) async {
      await tester.pumpWidget(SelectableDayPredicate());

      await tester.tap(find.byTooltip('Open navigation menu'));
      await tester.pumpAndSettle();
      await tester.tap(find.byKey(const Key('Reset')));
      await tester.pump();

      final dynamic count = find.byKey(const Key('predicateCount'));
      final Text predicateCount = count.evaluate().single.widget as Text;
      expect(
          predicateCount.data,
          'predicate count: 28 '
          'selection changed count: 0');
    });

    testWidgets('SelectableDayPredicate triggers count must be 0 for year view',
        (WidgetTester tester) async {
      await tester.pumpWidget(SelectableDayPredicate());

      await tester.tap(find.byTooltip('Open navigation menu'));
      await tester.pumpAndSettle();
      await tester.tap(find.byKey(const Key('year')));
      await tester.pump();
      await tester.tap(find.byKey(const Key('Reset')));
      await tester.pump();

      final dynamic count = find.byKey(const Key('predicateCount'));
      final Text predicateCount = count.evaluate().single.widget as Text;
      expect(
          predicateCount.data,
          'predicate count: 0 '
          'selection changed count: 0');
    });

    testWidgets(
        'SelectableDayPredicate triggers count must be 0 for decade view',
        (WidgetTester tester) async {
      await tester.pumpWidget(SelectableDayPredicate());

      await tester.tap(find.byTooltip('Open navigation menu'));
      await tester.pumpAndSettle();
      await tester.tap(find.byKey(const Key('decade')));
      await tester.pump();
      await tester.tap(find.byKey(const Key('Reset')));
      await tester.pump();

      final dynamic count = find.byKey(const Key('predicateCount'));
      final Text predicateCount = count.evaluate().single.widget as Text;
      expect(
          predicateCount.data,
          'predicate count: 0 '
          'selection changed count: 0');
    });

    testWidgets(
        'SelectableDayPredicate triggers count must be 0 for century view',
        (WidgetTester tester) async {
      await tester.pumpWidget(SelectableDayPredicate());

      await tester.tap(find.byTooltip('Open navigation menu'));
      await tester.pumpAndSettle();
      await tester.tap(find.byKey(const Key('century')));
      await tester.pump();
      await tester.tap(find.byKey(const Key('Reset')));
      await tester.pump();

      final dynamic count = find.byKey(const Key('predicateCount'));
      final Text predicateCount = count.evaluate().single.widget as Text;
      expect(
          predicateCount.data,
          'predicate count: 0 '
          'selection changed count: 0');
    });

    testWidgets(
        'SelectableDayPredicate triggers count must be 12 for year view',
        (WidgetTester tester) async {
      await tester.pumpWidget(SelectableDayPredicate());

      await tester.tap(find.byTooltip('Open navigation menu'));
      await tester.pumpAndSettle();
      await tester.tap(find.byKey(const Key('allowViewNavigation')));
      await tester.pump();
      await tester.tap(find.byKey(const Key('year')));
      await tester.pump();
      await tester.tap(find.byKey(const Key('Reset')));
      await tester.pump();

      final dynamic count = find.byKey(const Key('predicateCount'));
      final Text predicateCount = count.evaluate().single.widget as Text;
      expect(
          predicateCount.data,
          'predicate count: 12 '
          'selection changed count: 0');
    });

    testWidgets(
        'SelectableDayPredicate triggers count must be 12 for decade view',
        (WidgetTester tester) async {
      await tester.pumpWidget(SelectableDayPredicate());

      await tester.tap(find.byTooltip('Open navigation menu'));
      await tester.pumpAndSettle();
      await tester.tap(find.byKey(const Key('allowViewNavigation')));
      await tester.pump();
      await tester.tap(find.byKey(const Key('decade')));
      await tester.pump();
      await tester.tap(find.byKey(const Key('Reset')));
      await tester.pump();

      final dynamic count = find.byKey(const Key('predicateCount'));
      final Text predicateCount = count.evaluate().single.widget as Text;
      expect(
          predicateCount.data,
          'predicate count: 12 '
          'selection changed count: 0');
    });

    testWidgets(
        'SelectableDayPredicate triggers count must be 12 for century view',
        (WidgetTester tester) async {
      await tester.pumpWidget(SelectableDayPredicate());

      await tester.tap(find.byTooltip('Open navigation menu'));
      await tester.pumpAndSettle();
      await tester.tap(find.byKey(const Key('allowViewNavigation')));
      await tester.pump();
      await tester.tap(find.byKey(const Key('century')));
      await tester.pump();
      await tester.tap(find.byKey(const Key('Reset')));
      await tester.pump();

      final dynamic count = find.byKey(const Key('predicateCount'));
      final Text predicateCount = count.evaluate().single.widget as Text;
      expect(
          predicateCount.data,
          'predicate count: 12 '
          'selection changed count: 0');
    });

    testWidgets(
        'SelectableDayPredicate triggers count must be 12 for year view',
        (WidgetTester tester) async {
      await tester.pumpWidget(SelectableDayPredicate());

      await tester.tap(find.byTooltip('Open navigation menu'));
      await tester.pumpAndSettle();
      await tester.tap(find.byKey(const Key('year')));
      await tester.pump();
      await tester.tap(find.byKey(const Key('allowViewNavigation')));
      await tester.pump();
      await tester.tap(find.byKey(const Key('Reset')));
      await tester.pump();

      final dynamic count = find.byKey(const Key('predicateCount'));
      final Text predicateCount = count.evaluate().single.widget as Text;
      expect(
          predicateCount.data,
          'predicate count: 12 '
          'selection changed count: 0');
    });

    testWidgets(
        'SelectableDayPredicate triggers count must be 12 for decade view',
        (WidgetTester tester) async {
      await tester.pumpWidget(SelectableDayPredicate());

      await tester.tap(find.byTooltip('Open navigation menu'));
      await tester.pumpAndSettle();
      await tester.tap(find.byKey(const Key('decade')));
      await tester.pump();
      await tester.tap(find.byKey(const Key('allowViewNavigation')));
      await tester.pump();
      await tester.tap(find.byKey(const Key('Reset')));
      await tester.pump();

      final dynamic count = find.byKey(const Key('predicateCount'));
      final Text predicateCount = count.evaluate().single.widget as Text;
      expect(
          predicateCount.data,
          'predicate count: 12 '
          'selection changed count: 0');
    });

    testWidgets(
        'SelectableDayPredicate triggers count must be 12 for century view',
        (WidgetTester tester) async {
      await tester.pumpWidget(SelectableDayPredicate());

      await tester.tap(find.byTooltip('Open navigation menu'));
      await tester.pumpAndSettle();
      await tester.tap(find.byKey(const Key('century')));
      await tester.pump();
      await tester.tap(find.byKey(const Key('allowViewNavigation')));
      await tester.pump();
      await tester.tap(find.byKey(const Key('Reset')));
      await tester.pump();

      final dynamic count = find.byKey(const Key('predicateCount'));
      final Text predicateCount = count.evaluate().single.widget as Text;
      expect(
          predicateCount.data,
          'predicate count: 12 '
          'selection changed count: 0');
    });

    testWidgets(
        'SelectableDayPredicate triggers count must be 42 for month view',
        (WidgetTester tester) async {
      await tester.pumpWidget(SelectableDayPredicate());

      await tester.tap(find.byTooltip('Open navigation menu'));
      await tester.pumpAndSettle();
      await tester.tap(find.byKey(const Key('showTrailingAndLeadingDates')));
      await tester.pump();
      await tester.tap(find.byKey(const Key('Reset')));
      await tester.pump();

      final dynamic count = find.byKey(const Key('predicateCount'));
      final Text predicateCount = count.evaluate().single.widget as Text;
      expect(
          predicateCount.data,
          'predicate count: 42 '
          'selection changed count: 0');
    });

    testWidgets(
        'SelectableDayPredicate triggers count must be 59 for month view',
        (WidgetTester tester) async {
      await tester.pumpWidget(SelectableDayPredicate());

      await tester.tap(find.byTooltip('Open navigation menu'));
      await tester.pumpAndSettle();
      await tester.tap(find.byKey(const Key('hideTrailingAndLeadingDates')));
      await tester.pump();
      await tester.tap(find.byKey(const Key('enableMultiView')));
      await tester.pump();
      await tester.tap(find.byKey(const Key('Reset')));
      await tester.pump();

      final dynamic count = find.byKey(const Key('predicateCount'));
      final Text predicateCount = count.evaluate().single.widget as Text;
      expect(
          predicateCount.data,
          'predicate count: 59 '
          'selection changed count: 0');
    });

    testWidgets(
        'SelectableDayPredicate triggers count must be 24 for year view',
        (WidgetTester tester) async {
      await tester.pumpWidget(SelectableDayPredicate());

      await tester.tap(find.byTooltip('Open navigation menu'));
      await tester.pumpAndSettle();
      await tester.tap(find.byKey(const Key('allowViewNavigation')));
      await tester.pump();
      await tester.tap(find.byKey(const Key('year')));
      await tester.pump();
      await tester.tap(find.byKey(const Key('enableMultiView')));
      await tester.pump();
      await tester.tap(find.byKey(const Key('Reset')));
      await tester.pump();

      final dynamic count = find.byKey(const Key('predicateCount'));
      final Text predicateCount = count.evaluate().single.widget as Text;
      expect(
          predicateCount.data,
          'predicate count: 24 '
          'selection changed count: 0');
    });

    testWidgets(
        'SelectableDayPredicate triggers count must be 24 for decade view',
        (WidgetTester tester) async {
      await tester.pumpWidget(SelectableDayPredicate());

      await tester.tap(find.byTooltip('Open navigation menu'));
      await tester.pumpAndSettle();
      await tester.tap(find.byKey(const Key('allowViewNavigation')));
      await tester.pump();
      await tester.tap(find.byKey(const Key('decade')));
      await tester.pump();
      await tester.tap(find.byKey(const Key('enableMultiView')));
      await tester.pump();
      await tester.tap(find.byKey(const Key('Reset')));
      await tester.pump();

      final dynamic count = find.byKey(const Key('predicateCount'));
      final Text predicateCount = count.evaluate().single.widget as Text;
      expect(
          predicateCount.data,
          'predicate count: 24 '
          'selection changed count: 0');
    });

    testWidgets(
        'SelectableDayPredicate triggers count must be 24 for century view',
        (WidgetTester tester) async {
      await tester.pumpWidget(SelectableDayPredicate());

      await tester.tap(find.byTooltip('Open navigation menu'));
      await tester.pumpAndSettle();
      await tester.tap(find.byKey(const Key('allowViewNavigation')));
      await tester.pump();
      await tester.tap(find.byKey(const Key('enableMultiView')));
      await tester.pump();
      await tester.tap(find.byKey(const Key('century')));
      await tester.pump();
      await tester.tap(find.byKey(const Key('Reset')));
      await tester.pump();

      final dynamic count = find.byKey(const Key('predicateCount'));
      final Text predicateCount = count.evaluate().single.widget as Text;
      expect(
          predicateCount.data,
          'predicate count: 24 '
          'selection changed count: 0');
    });

    testWidgets(
        'SelectableDayPredicate triggers count must be 84 for month view',
        (WidgetTester tester) async {
      await tester.pumpWidget(SelectableDayPredicate());

      await tester.tap(find.byTooltip('Open navigation menu'));
      await tester.pumpAndSettle();
      await tester.tap(find.byKey(const Key('showTrailingAndLeadingDates')));
      await tester.pump();
      await tester.tap(find.byKey(const Key('enableMultiView')));
      await tester.pump();
      await tester.tap(find.byKey(const Key('Reset')));
      await tester.pump();

      final dynamic count = find.byKey(const Key('predicateCount'));
      final Text predicateCount = count.evaluate().single.widget as Text;
      expect(
          predicateCount.data,
          'predicate count: 84 '
          'selection changed count: 0');
    });
  });
}

void hijriSelectablePredicates() {
  group('HijriSelectableDayPredicate', () {
    testWidgets(
        'HijriSelectableDayPredicate triggers count must be 29 for month view',
        (WidgetTester tester) async {
      await tester.pumpWidget(HijriSelectablePredicate());

      await tester.tap(find.byTooltip('Open navigation menu'));
      await tester.pumpAndSettle();
      await tester.tap(find.byKey(const Key('Reset')));
      await tester.pump();

      final dynamic count = find.byKey(const Key('predicateCount'));
      final Text predicateCount = count.evaluate().single.widget as Text;
      expect(
          predicateCount.data,
          'predicate count: 29 '
          'selection changed count: 0');
    });

    testWidgets(
        'HijriSelectableDayPredicate triggers count must be 0 for year view',
        (WidgetTester tester) async {
      await tester.pumpWidget(HijriSelectablePredicate());

      await tester.tap(find.byTooltip('Open navigation menu'));
      await tester.pumpAndSettle();
      await tester.tap(find.byKey(const Key('year')));
      await tester.pump();
      await tester.tap(find.byKey(const Key('Reset')));
      await tester.pump();

      final dynamic count = find.byKey(const Key('predicateCount'));
      final Text predicateCount = count.evaluate().single.widget as Text;
      expect(
          predicateCount.data,
          'predicate count: 0 '
          'selection changed count: 0');
    });

    testWidgets(
        'HijriSelectableDayPredicate triggers count must be 0 for decade view',
        (WidgetTester tester) async {
      await tester.pumpWidget(HijriSelectablePredicate());

      await tester.tap(find.byTooltip('Open navigation menu'));
      await tester.pumpAndSettle();
      await tester.tap(find.byKey(const Key('decade')));
      await tester.pump();
      await tester.tap(find.byKey(const Key('Reset')));
      await tester.pump();

      final dynamic count = find.byKey(const Key('predicateCount'));
      final Text predicateCount = count.evaluate().single.widget as Text;
      expect(
          predicateCount.data,
          'predicate count: 0 '
          'selection changed count: 0');
    });

    testWidgets(
        'HijriSelectableDayPredicate triggers count must be 12 for year view',
        (WidgetTester tester) async {
      await tester.pumpWidget(HijriSelectablePredicate());

      await tester.tap(find.byTooltip('Open navigation menu'));
      await tester.pumpAndSettle();
      await tester.tap(find.byKey(const Key('allowViewNavigation')));
      await tester.pump();
      await tester.tap(find.byKey(const Key('year')));
      await tester.pump();
      await tester.tap(find.byKey(const Key('Reset')));
      await tester.pump();

      final dynamic count = find.byKey(const Key('predicateCount'));
      final Text predicateCount = count.evaluate().single.widget as Text;
      expect(
          predicateCount.data,
          'predicate count: 12 '
          'selection changed count: 0');
    });

    testWidgets(
        'HijriSelectableDayPredicate triggers count must be 12 for decade view',
        (WidgetTester tester) async {
      await tester.pumpWidget(HijriSelectablePredicate());

      await tester.tap(find.byTooltip('Open navigation menu'));
      await tester.pumpAndSettle();
      await tester.tap(find.byKey(const Key('allowViewNavigation')));
      await tester.pump();
      await tester.tap(find.byKey(const Key('decade')));
      await tester.pump();
      await tester.tap(find.byKey(const Key('Reset')));
      await tester.pump();

      final dynamic count = find.byKey(const Key('predicateCount'));
      final Text predicateCount = count.evaluate().single.widget as Text;
      expect(
          predicateCount.data,
          'predicate count: 12 '
          'selection changed count: 0');
    });

    testWidgets(
        'HijriSelectableDayPredicate triggers count must be 12 for year view',
        (WidgetTester tester) async {
      await tester.pumpWidget(HijriSelectablePredicate());

      await tester.tap(find.byTooltip('Open navigation menu'));
      await tester.pumpAndSettle();
      await tester.tap(find.byKey(const Key('year')));
      await tester.pump();
      await tester.tap(find.byKey(const Key('allowViewNavigation')));
      await tester.pump();
      await tester.tap(find.byKey(const Key('Reset')));
      await tester.pump();

      final dynamic count = find.byKey(const Key('predicateCount'));
      final Text predicateCount = count.evaluate().single.widget as Text;
      expect(
          predicateCount.data,
          'predicate count: 12 '
          'selection changed count: 0');
    });

    testWidgets(
        'HijriSelectableDayPredicate triggers count must be 12 for decade view',
        (WidgetTester tester) async {
      await tester.pumpWidget(HijriSelectablePredicate());

      await tester.tap(find.byTooltip('Open navigation menu'));
      await tester.pumpAndSettle();

      await tester.tap(find.byKey(const Key('decade')));
      await tester.pump();
      await tester.tap(find.byKey(const Key('allowViewNavigation')));
      await tester.pump();
      await tester.tap(find.byKey(const Key('Reset')));
      await tester.pump();

      final dynamic count = find.byKey(const Key('predicateCount'));
      final Text predicateCount = count.evaluate().single.widget as Text;
      expect(
          predicateCount.data,
          'predicate count: 12 '
          'selection changed count: 0');
    });

    testWidgets(
        'Multi view - HijriSelectableDayPredicate triggers count must be 59 '
        'for month view', (WidgetTester tester) async {
      await tester.pumpWidget(HijriSelectablePredicate());

      await tester.tap(find.byTooltip('Open navigation menu'));
      await tester.pumpAndSettle();
      await tester.tap(find.byKey(const Key('enableMultiView')));
      await tester.pump();
      await tester.tap(find.byKey(const Key('Reset')));
      await tester.pump();

      final dynamic count = find.byKey(const Key('predicateCount'));
      final Text predicateCount = count.evaluate().single.widget as Text;
      expect(
          predicateCount.data,
          'predicate count: 59 '
          'selection changed count: 0');
    });

    testWidgets(
        'Multi view - HijriSelectableDayPredicate triggers count must be 24 '
        'for year view', (WidgetTester tester) async {
      await tester.pumpWidget(HijriSelectablePredicate());

      await tester.tap(find.byTooltip('Open navigation menu'));
      await tester.pumpAndSettle();
      await tester.tap(find.byKey(const Key('allowViewNavigation')));
      await tester.pump();
      await tester.tap(find.byKey(const Key('year')));
      await tester.pump();
      await tester.tap(find.byKey(const Key('enableMultiView')));
      await tester.pump();
      await tester.tap(find.byKey(const Key('Reset')));
      await tester.pump();

      final dynamic count = find.byKey(const Key('predicateCount'));
      final Text predicateCount = count.evaluate().single.widget as Text;
      expect(
          predicateCount.data,
          'predicate count: 24 '
          'selection changed count: 0');
    });

    testWidgets(
        'Multi view - HijriSelectableDayPredicate triggers count must be 24 '
        'for decade view', (WidgetTester tester) async {
      await tester.pumpWidget(HijriSelectablePredicate());

      await tester.tap(find.byTooltip('Open navigation menu'));
      await tester.pumpAndSettle();
      await tester.tap(find.byKey(const Key('allowViewNavigation')));
      await tester.pump();
      await tester.tap(find.byKey(const Key('decade')));
      await tester.pump();
      await tester.tap(find.byKey(const Key('enableMultiView')));
      await tester.pump();
      await tester.tap(find.byKey(const Key('Reset')));
      await tester.pump();

      final dynamic count = find.byKey(const Key('predicateCount'));
      final Text predicateCount = count.evaluate().single.widget as Text;
      expect(
          predicateCount.data,
          'predicate count: 24 '
          'selection changed count: 0');
    });
  });
}

/// Test method to test and ensure the date time engine in date range picker.
void dateTimeEngineTest() {
  group('visible dates count', () {
    test('visible dates count must be 42 for month view', () {
      final List<DateTime> visibleDates =
          getVisibleDates(DateTime.now(), null, 1, 42).cast<DateTime>();
      expect(visibleDates.length, 42);
    });
  });

  group('first day of week for month view', () {
    test('First day must be monday', () {
      final List<DateTime> visibleDates =
          getVisibleDates(DateTime(2019, 02, 05), null, 1, 42).cast<DateTime>();
      expect(visibleDates[0], DateTime(2019, 1, 28));
      expect(visibleDates[0].weekday, DateTime.monday);
    });

    test('First day must be tuesday', () {
      final List<DateTime> visibleDates =
          getVisibleDates(DateTime(2019, 02, 05), null, 2, 42).cast<DateTime>();
      expect(visibleDates[0], DateTime(2019, 1, 29));
      expect(visibleDates[0].weekday, DateTime.tuesday);
    });

    test('First day must be wednesday', () {
      final List<DateTime> visibleDates =
          getVisibleDates(DateTime(2019, 02, 05), null, 3, 42).cast<DateTime>();
      expect(visibleDates[0], DateTime(2019, 1, 30));
      expect(visibleDates[0].weekday, DateTime.wednesday);
    });

    test('First day must be thursday', () {
      final List<DateTime> visibleDates =
          getVisibleDates(DateTime(2019, 02, 05), null, 4, 42).cast<DateTime>();
      expect(visibleDates[0], DateTime(2019, 1, 31));
      expect(visibleDates[0].weekday, DateTime.thursday);
    });

    test('First day must be friday', () {
      final List<DateTime> visibleDates =
          getVisibleDates(DateTime(2019, 02, 05), null, 5, 42).cast<DateTime>();
      expect(visibleDates[0], DateTime(2019, 2));
      expect(visibleDates[0].weekday, DateTime.friday);
    });

    test('First day must be saturday', () {
      final List<DateTime> visibleDates =
          getVisibleDates(DateTime(2019, 02, 05), null, 6, 42).cast<DateTime>();
      expect(visibleDates[0], DateTime(2019, 1, 26));
      expect(visibleDates[0].weekday, DateTime.saturday);
    });

    test('First day must be sunday', () {
      final List<DateTime> visibleDates =
          getVisibleDates(DateTime(2019, 02, 05), null, 7, 42).cast<DateTime>();
      expect(visibleDates[0], DateTime(2019, 1, 27));
      expect(visibleDates[0].weekday, DateTime.sunday);
    });
  });

  test('visible dates count must be 4', () {
    final List<DateTime> visibleDates = getVisibleDates(DateTime(2019, 02, 05),
            <int>[DateTime.saturday, DateTime.sunday, DateTime.monday], 6, 4)
        .cast<DateTime>();
    expect(visibleDates.length, 4);
    expect(visibleDates[0].weekday, DateTime.tuesday);
    expect(visibleDates[1].weekday, DateTime.wednesday);
    expect(visibleDates[2].weekday, DateTime.thursday);
    expect(visibleDates[3].weekday, DateTime.friday);
  });

  test('visible dates count must be 6', () {
    final List<DateTime> visibleDates = getVisibleDates(
            DateTime(2019, 02, 05),
            <int>[
              DateTime.sunday,
            ],
            6,
            6)
        .cast<DateTime>();
    expect(visibleDates.length, 6);
    expect(visibleDates[0].weekday, DateTime.saturday);
    expect(visibleDates[1].weekday, DateTime.monday);
    expect(visibleDates[2].weekday, DateTime.tuesday);
    expect(visibleDates[3].weekday, DateTime.wednesday);
    expect(visibleDates[4].weekday, DateTime.thursday);
    expect(visibleDates[5].weekday, DateTime.friday);
  });

  group('Number of rows in view test', () {
    test('visible dates must be 14', () {
      final List<DateTime> visibleDates =
          getVisibleDates(DateTime(2019, 02, 05), null, 7, 14).cast<DateTime>();
      expect(visibleDates.length, 14);
      expect(visibleDates[0], DateTime(2019, 02, 03));
    });

    test('visible dates must be 7', () {
      final List<DateTime> visibleDates =
          getVisibleDates(DateTime(2019, 02, 05), null, 7, 7).cast<DateTime>();
      expect(visibleDates.length, 7);
      expect(visibleDates[0], DateTime(2019, 02, 03));
    });

    test('visible dates must be 35', () {
      final List<DateTime> visibleDates =
          getVisibleDates(DateTime(2019, 02, 05), null, 7, 35).cast<DateTime>();
      expect(visibleDates.length, 35);
      expect(visibleDates[0], DateTime(2019, 02, 03));
    });
  });

  group('First day of week value with number of rows in view', () {
    test('First day of the week must be monday', () {
      final List<DateTime> visibleDates =
          getVisibleDates(DateTime(2019, 02, 27), null, 1, 14).cast<DateTime>();
      expect(visibleDates[0].weekday, DateTime.monday);
      expect(visibleDates[0], DateTime(2019, 02, 25));
      expect(visibleDates.length, 14);
    });

    test('First day of the week must be tuesday', () {
      final List<DateTime> visibleDates =
          getVisibleDates(DateTime(2019, 02, 27), null, 2, 14).cast<DateTime>();
      expect(visibleDates[0].weekday, DateTime.tuesday);
      expect(visibleDates[0], DateTime(2019, 02, 26));
      expect(visibleDates.length, 14);
    });

    test('First day of the week must be wednesday', () {
      final List<DateTime> visibleDates =
          getVisibleDates(DateTime(2019, 02, 27), null, 3, 14).cast<DateTime>();
      expect(visibleDates[0].weekday, DateTime.wednesday);
      expect(visibleDates[0], DateTime(2019, 02, 27));
      expect(visibleDates.length, 14);
    });

    test('First day of the week must be thursday', () {
      final List<DateTime> visibleDates =
          getVisibleDates(DateTime(2019, 02, 27), null, 4, 14).cast<DateTime>();
      expect(visibleDates[0].weekday, DateTime.thursday);
      expect(visibleDates[0], DateTime(2019, 02, 21));
      expect(visibleDates.length, 14);
    });

    test('First day of the week must be friday', () {
      final List<DateTime> visibleDates =
          getVisibleDates(DateTime(2019, 02, 27), null, 5, 14).cast<DateTime>();
      expect(visibleDates[0].weekday, DateTime.friday);
      expect(visibleDates[0], DateTime(2019, 02, 22));
      expect(visibleDates.length, 14);
    });

    test('First day of the week must be saturday', () {
      final List<DateTime> visibleDates =
          getVisibleDates(DateTime(2019, 02, 27), null, 6, 14).cast<DateTime>();
      expect(visibleDates[0].weekday, DateTime.saturday);
      expect(visibleDates[0], DateTime(2019, 02, 23));
      expect(visibleDates.length, 14);
    });

    test('First day of the week must be sunday', () {
      final List<DateTime> visibleDates =
          getVisibleDates(DateTime(2019, 02, 27), null, 7, 14).cast<DateTime>();
      expect(visibleDates[0].weekday, DateTime.sunday);
      expect(visibleDates[0], DateTime(2019, 02, 24));
      expect(visibleDates.length, 14);
    });
  });

  group('Testing the next and previous month value', () {
    test('Next month value must be 1', () {
      final DateTime nextMonth = DateRangePickerHelper.getDateTimeValue(
          getNextMonthDate(DateTime(2019, 12, 15)));
      expect(nextMonth.month, DateTime.january);
      expect(nextMonth, DateTime(2020));
    });

    test('Prevous month value must be 12', () {
      final DateTime previousMonth = DateRangePickerHelper.getDateTimeValue(
          getPreviousMonthDate(DateTime(2019, 01, 15)));
      expect(previousMonth.month, DateTime.december);
      expect(previousMonth, DateTime(2018, 12));
    });

    test('Next month value must be 3', () {
      final DateTime nextMonth = DateRangePickerHelper.getDateTimeValue(
          getNextMonthDate(DateTime(2019, 02, 05)));
      expect(nextMonth.month, DateTime.march);
      expect(nextMonth, DateTime(2019, 03));
    });

    test('Previous month value must be 4', () {
      final DateTime nextMonth = DateRangePickerHelper.getDateTimeValue(
          getPreviousMonthDate(DateTime(2019, 05, 05)));
      expect(nextMonth.month, DateTime.april);
      expect(nextMonth, DateTime(2019, 04));
    });
  });
}

/// The method to ensure the hijri calendar conversion for date.
void hijriCalendarHelperTest() {
  group('convert Jan date', () {
    test('converted date must be 20th date and 5th month of 1441', () {
      final HijriDateTime hijriDateDetails =
          convertToHijriDate(DateTime(2020, 01, 15));
      expect(hijriDateDetails.year, 1441);
      expect(hijriDateDetails.month, 5);
      expect(hijriDateDetails.day, 20);
      final DateTime gregorianDate =
          convertToGregorianDate(null, year: 1441, month: 5, day: 20);
      expect(gregorianDate.year, 2020);
      expect(gregorianDate.month, 1);
      expect(gregorianDate.day, 15);
    });
  });
  group('convert Feb date', () {
    test('converted date must be 11th date and 6th month of 1441', () {
      final HijriDateTime hijriDateDetails =
          convertToHijriDate(DateTime(2020, 02, 05));

      expect(hijriDateDetails.year, 1441);
      expect(hijriDateDetails.month, 6);
      expect(hijriDateDetails.day, 11);

      final DateTime gregorianDate =
          convertToGregorianDate(null, year: 1441, month: 6, day: 11);
      expect(gregorianDate.year, 2020);
      expect(gregorianDate.month, 2);
      expect(gregorianDate.day, 5);
    });
  });
  group('convert March date', () {
    test('converted date must be 2st date and 8th month of 1441', () {
      final HijriDateTime hijriDateDetails =
          convertToHijriDate(DateTime(2020, 03, 25));

      expect(hijriDateDetails.year, 1441);
      expect(hijriDateDetails.month, 8);
      expect(hijriDateDetails.day, 1);
      final DateTime gregorianDate =
          convertToGregorianDate(null, year: 1441, month: 8, day: 1);
      expect(gregorianDate.year, 2020);
      expect(gregorianDate.month, 3);
      expect(gregorianDate.day, 25);
    });
  });
  group('convert April date', () {
    test('converted date must be 8st date and 8th month of 1441', () {
      final HijriDateTime hijriDateDetails =
          convertToHijriDate(DateTime(2020, 04));

      expect(hijriDateDetails.year, 1441);
      expect(hijriDateDetails.month, 8);
      expect(hijriDateDetails.day, 8);

      final DateTime gregorianDate =
          convertToGregorianDate(null, year: 1441, month: 8, day: 8);
      expect(gregorianDate.year, 2020);
      expect(gregorianDate.month, 4);
      expect(gregorianDate.day, 1);
    });
  });
  group('convert May date', () {
    test('converted date must be 22nd date and 9th month of 1441', () {
      final HijriDateTime hijriDateDetails =
          convertToHijriDate(DateTime(2020, 05, 22));

      expect(hijriDateDetails.year, 1441);
      expect(hijriDateDetails.month, 9);
      expect(hijriDateDetails.day, 29);

      final DateTime gregorianDate =
          convertToGregorianDate(null, year: 1441, month: 9, day: 29);
      expect(gregorianDate.year, 2020);
      expect(gregorianDate.month, 5);
      expect(gregorianDate.day, 22);
    });
  });
  group('convert June date', () {
    test('converted date must be 25th date and 10th month of 1441', () {
      final HijriDateTime hijriDateDetails =
          convertToHijriDate(DateTime(2020, 06, 17));

      expect(hijriDateDetails.year, 1441);
      expect(hijriDateDetails.month, 10);
      expect(hijriDateDetails.day, 25);

      final DateTime gregorianDate =
          convertToGregorianDate(null, year: 1441, month: 10, day: 25);
      expect(gregorianDate.year, 2020);
      expect(gregorianDate.month, 6);
      expect(gregorianDate.day, 17);
    });
  });
  group('convert July date', () {
    test('converted date must be 6th date and 12th month of 1441', () {
      final HijriDateTime hijriDateDetails =
          convertToHijriDate(DateTime(2020, 07, 27));

      expect(hijriDateDetails.year, 1441);
      expect(hijriDateDetails.month, 12);
      expect(hijriDateDetails.day, 6);

      final DateTime gregorianDate =
          convertToGregorianDate(null, year: 1441, month: 12, day: 6);
      expect(gregorianDate.year, 2020);
      expect(gregorianDate.month, 7);
      expect(gregorianDate.day, 27);
    });
  });
  group('convert August date', () {
    test('converted date must be 25th date and 12th month of 1441', () {
      final HijriDateTime hijriDateDetails =
          convertToHijriDate(DateTime(2020, 08, 15));

      expect(hijriDateDetails.year, 1441);
      expect(hijriDateDetails.month, 12);
      expect(hijriDateDetails.day, 25);

      final DateTime gregorianDate =
          convertToGregorianDate(null, year: 1441, month: 12, day: 25);
      expect(gregorianDate.year, 2020);
      expect(gregorianDate.month, 8);
      expect(gregorianDate.day, 15);
    });
  });
  group('convert Sep date', () {
    test('converted date must be 20th date and 1th month of 1442', () {
      final HijriDateTime hijriDateDetails =
          convertToHijriDate(DateTime(2020, 09, 08));

      expect(hijriDateDetails.year, 1442);
      expect(hijriDateDetails.month, 1);
      expect(hijriDateDetails.day, 20);

      final DateTime gregorianDate =
          convertToGregorianDate(null, year: 1442, month: 1, day: 20);
      expect(gregorianDate.year, 2020);
      expect(gregorianDate.month, 9);
      expect(gregorianDate.day, 08);
    });
  });

  group('converted October date', () {
    test('converted date must be 29th date and 2nd month of 1442', () {
      final HijriDateTime hijriDateDetails =
          convertToHijriDate(DateTime(2020, 10, 16));

      expect(hijriDateDetails.year, 1442);
      expect(hijriDateDetails.month, 2);
      expect(hijriDateDetails.day, 29);

      final DateTime gregorianDate =
          convertToGregorianDate(null, year: 1442, month: 2, day: 29);
      expect(gregorianDate.year, 2020);
      expect(gregorianDate.month, 10);
      expect(gregorianDate.day, 16);
    });
  });
  group('converted November date', () {
    test('converted date must be 29th date and 2nd month of 1442', () {
      final HijriDateTime hijriDateDetails =
          convertToHijriDate(DateTime(2020, 11, 28));

      expect(hijriDateDetails.year, 1442);
      expect(hijriDateDetails.month, 4);
      expect(hijriDateDetails.day, 13);

      final DateTime gregorianDate =
          convertToGregorianDate(null, year: 1442, month: 4, day: 13);
      expect(gregorianDate.year, 2020);
      expect(gregorianDate.month, 11);
      expect(gregorianDate.day, 28);
    });
  });
  group('converted December date', () {
    test('converted date must be 29th date and 2nd month of 1442', () {
      final HijriDateTime hijriDateDetails =
          convertToHijriDate(DateTime(2020, 12, 09));

      expect(hijriDateDetails.year, 1442);
      expect(hijriDateDetails.month, 4);
      expect(hijriDateDetails.day, 24);

      final DateTime gregorianDate =
          convertToGregorianDate(null, year: 1442, month: 4, day: 24);
      expect(gregorianDate.year, 2020);
      expect(gregorianDate.month, 12);
      expect(gregorianDate.day, 9);
    });
  });
}
